#ifndef EVENT_CONFLICT_TABLE_H_INCLUDED_
#define EVENT_CONFLICT_TABLE_H_INCLUDED_

#include "rose.h"

#include "conflict_table.h"
#include "mapped_variable.h"
#include "port_call_path.h"

#include <unordered_map>


namespace risc {

class PathInstanceMapper;

namespace sg {

class SegmentGraph;

/**
 * \class EventConflictTable
 * \brief The conflicts are stored in the base class variable conflict_table_.
 *        The columns represent wait statements of the segments. The rows are
 *        representing the notifications of events. So, a true in
 *        conflict_table_[i][j] means that the associated segment/instance i
 *        notifies the event for which the associated segment/instance j is
 *        waiting.
 *
 * TODO   Handling for pointers
 */
class EventConflictTable: public ConflictTable {

public:

  /**
   * \brief    Default constructor for the DataConflictTable
   */
  EventConflictTable(
      SegmentGraph &graph,
      PathInstanceMapper *path_instance_mapper,
      bool debugging);

  /**
   * \fn       get_all_list_events_statically(SgSymbol *sc_event_list, int instance_id, PortCallPath pcp)
   * \brief    This function determines all the registered events in the event
   *           list statically
   */
  std::list<risc::sg::MappedVariable>
  get_all_list_events_statically(SgSymbol *sc_event_list, int instance_id, PortCallPath pcp);

  /**
   * \brief    added to support immediate notification and notify with time
   */
  std::unordered_map<int, std::unordered_map<int, Time>> notify_delay_;
  /**
   * \fn       get_all_list_events_dynamically(
      SgVariableSymbol *sc_event_list,
      int instance_id)
   * \brief    This function determines all the registered events in the event
   *           list dynamically
   */
  std::list<risc::sg::MappedVariable>
  get_all_list_events_dynamically(
      SgVariableSymbol *sc_event_list,
      int instance_id);

  /**
   * \fn       determine_conflict_table
   * \brief    This function determines conflicts among the segments
   */
  virtual void determine_conflict_table();

  /**
   * \fn       determine_mapped_events(
      std::set<SymbolWithPath> &event_list,
      int instance_id,
      bool dynamic_analysis)
   * \brief    This function determines the mapping of events
   *           (RD, 07/13/18)
   */
  std::list<risc::sg::MappedVariable>
  determine_mapped_events(
      std::set<SymbolWithPath> &event_list,
      int instance_id,
      bool dynamic_analysis);
};

}; // end of namesapce sg

}; // end of namespace risc


#endif /* EVENT_CONFLICT_TABLE_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
