#ifndef FUNCTION_ANNOTATION_ATTRIBUTES_H_INCLUDED_
#define FUNCTION_ANNOTATION_ATTRIBUTES_H_INCLUDED_

#include "rose.h"

#include "function_annotation.h"

namespace risc {

namespace sg {

/**
 *  \class FunctionAnnotationAttributes
 *  \brief This class provides attributes for function anotations
 *
 */
class FunctionAnnotationAttributes: public AstAttribute {

public:

  FunctionAnnotationAttributes(FunctionAnnotationAttributes *func_attributes);

  FunctionAnnotationAttributes(FunctionAnnotation *annotation);

  // for the new rose release
  virtual AstAttribute::OwnershipPolicy getOwnershipPolicy() const ROSE_OVERRIDE
  {
    return CONTAINER_OWNERSHIP;
  }


  FunctionAnnotationAttributes(
    std::string function_name,
    bool is_conflict_free,
    WAIT_CONSTRUCT wait_type,
    AnnotatedTime *annotated_time);
  
  /**
   * \fn        has_boundary
   * \brief     Shows if this function contains a boundary (wait statements)
   */
  bool has_boundary();

  /**
   * \brief Assigned duing annotation analysis
   */
  std::string function_name_;

  /**
   * \brief Function does not have any conflict with other segments
   */
  bool is_conflict_free_;

  /**
   * \brief wait_type_
   *        Wait type of this function
   */
  WAIT_CONSTRUCT wait_type_;

  /**
   * \brief Represents the wait time
   */
  AnnotatedTime *annotated_time_;

  bool operator==(const FunctionAnnotationAttributes &other);

  bool operator!=(const FunctionAnnotationAttributes &other);
};

} // end of namespace sg

} // end of namespace risc

#endif /* FUNCTION_ANNOTATION_ATTRIBUTES_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
