#include "tlm.h"                                    // TLM headers
#include "tlm_utils/simple_initiator_socket.h"      // TLM simple initiator socket
#include "tlm_utils/simple_target_socket.h"
#include <string>
#include "systemc.h"
using namespace sc_core;
using namespace std;

class lt_initiator :  public sc_core::sc_module 
{
public:
    lt_initiator( sc_core::sc_module_name name);
  
  	void initiator_thread (void);                    
  
    SC_HAS_PROCESS(lt_initiator);
   
    tlm_utils::simple_initiator_socket<lt_initiator> initiator_socket;
 
    tlm::tlm_response_status gp_status;
}; 


class lt_target : public sc_core::sc_module           	
{
public:	  
 
	lt_target( sc_core::sc_module_name   module_name);

	void custom_b_transport( 
		tlm::tlm_generic_payload  &gp,
		sc_core::sc_time          &delay_time
  	);

	SC_HAS_PROCESS(lt_target);
	tlm_utils::simple_target_socket<lt_target>  m_memory_socket;

	std::string              m_target_memory;
};

class lt_top : public sc_core::sc_module                   
{
public:
		
	lt_top ( sc_core::sc_module_name name); 
	//SimpleBusLT<1, 1>       m_bus;                ///< simple bus
	lt_target               m_lt_target_1;          ///< blocking with convenienece socket
	lt_initiator            m_initiator_1;          ///< instance 1 initiator
};

lt_initiator::lt_initiator( sc_module_name name )
: sc_module           (name)                      // initialize module name
, initiator_socket    ("initiator_socket")        // initiator socket
{                
	SC_THREAD(initiator_thread);                  
}

void lt_initiator::initiator_thread(void)   ///< initiator thread
{  
	tlm::tlm_generic_payload trans; 
	unsigned char data_buffer[11+1] = { 0 };

	trans.set_data_ptr(data_buffer);
	trans.set_command(tlm::TLM_READ_COMMAND);
	trans.set_address( 0 );
	trans.set_data_length( 11 ); 


	sc_time delay = SC_ZERO_TIME;       // Create delay objects 

	initiator_socket->b_transport(trans, delay);

	gp_status = trans.get_response_status();

	if(gp_status == tlm::TLM_OK_RESPONSE)
	{
	std::cout << "received" << std::endl;
	std::cout << data_buffer << std::endl;
	}
} 

///Constructor
lt_target::lt_target( sc_core::sc_module_name module_name )
: sc_module (module_name)
, m_memory_socket ("memory_socket_1")
, m_target_memory ("Hello World")
{
  m_memory_socket.register_b_transport(this, &lt_target::custom_b_transport);
}

void                                        
lt_target::custom_b_transport( 
	tlm::tlm_generic_payload  &gp,
	sc_core::sc_time          &delay_time             // delay time 
)
{
  
  int address = gp.get_address();
  tlm::tlm_command command = gp.get_command();
  unsigned char *data = gp.get_data_ptr();
  int length = gp.get_data_length();
  wait(1,SC_NS);
  switch (command)
  {
    default:
    {
    
      std::cout << " Unsupported Command Extension" << std::endl;
      gp.set_response_status(tlm::TLM_COMMAND_ERROR_RESPONSE);
      delay_time = sc_core::SC_ZERO_TIME;
    
      break;
    }
    

    case tlm::TLM_READ_COMMAND:
    {
      gp.set_response_status(tlm::TLM_OK_RESPONSE);
    
      for (unsigned int i = 0; i < length; i++)
      {
        data[i] = m_target_memory[address++];
        delay_time = sc_time(10,SC_NS);  
        std::cout << "reading" << std::endl;       
      }
      
       
      break;
    }
  } // end switch
  
  return;     
}

lt_top::lt_top( sc_core::sc_module_name name )
: sc_core::sc_module(name)
, m_lt_target_1("m_lt_target_1")
, m_initiator_1("m_initiator_1")
{
  std::cout << "in top.cpp" << std::endl;
  m_initiator_1.initiator_socket(m_lt_target_1.m_memory_socket);
}

int sc_main(int   argc, char  *argv[] )
{
  lt_top top("top");                    // instantiate a exmaple top module

  sc_core::sc_start();                  // start the simulation

  return 0;                             // return okay status
}
