#include "systemc.h"

typedef struct Particles_t
{
  Particles_t():
    x(0)
  { }

  int x;

  friend std::ostream& operator<<(std::ostream& os, const Particles_t& co);
} particle_t;

class i_particle_transceiver: virtual public sc_interface
{
public:
  virtual void read(particle_t) = 0;
  virtual void write(const particle_t) = 0;
};

class c_particle_double_handshake_channel:
  public sc_channel, public i_particle_transceiver
{
public:
  c_particle_double_handshake_channel(sc_module_name name):
    sc_channel(name)
  {
    v = false;
    w = false;
  }

  ~c_particle_double_handshake_channel()
  { }

  void read(particle_t Y)
  {
    if(!v) {
      w = true;
      wait(req);
      w = false;
    }
    Y = tmpd;
    v = false;
    ack.notify(SC_ZERO_TIME);
    wait(ack);
  }

  void write(const particle_t X)
  {
    tmpd = X;
    v = true;
    if(w) {
      req.notify(SC_ZERO_TIME);
   }
   wait(ack);
  }

private:
  sc_event req;
  sc_event ack;
  bool v, w;
  particle_t tmpd;
};

class Sender: public sc_module {

public:

  SC_HAS_PROCESS(Sender);

  sc_port<i_particle_transceiver> port1;
  sc_port<i_particle_transceiver> port2;

  Sender(sc_module_name mn):
    sc_module(mn)
  {
    SC_THREAD(sender_custom);
  }

  void sender_custom()
  {
    particle_t particle;
    particle.x = 42;
    port1->write(particle);

    int c;
    wait(SC_ZERO_TIME);
    int d;
  }
};

class Receiver: public sc_module {

public:

  SC_HAS_PROCESS(Receiver);

  sc_port<i_particle_transceiver> port1;
  sc_port<i_particle_transceiver> port2;

  Receiver(sc_module_name mn):
    sc_module(mn)
  {
    SC_THREAD(receiver_custom);
  }

  void receiver_custom()
  {
    int a;
    wait(SC_ZERO_TIME);
    int b;

    particle_t particle;

    port1->read(particle);

    int c;
    wait(SC_ZERO_TIME);
    int d;
  }
};

class Top1: public sc_module {

public:
  Sender sender;
  Receiver receiver;
  c_particle_double_handshake_channel channel1;
  c_particle_double_handshake_channel channel2;

  Top1(sc_module_name mn):
    sc_module(mn),
    sender("sender"),
    receiver("receiver"),
    channel1("channel1"),
    channel2("channel2")
  { }

  void before_end_of_elaboration()
  {
    sender.port1.bind(channel1);
    receiver.port1.bind(channel1);

    sender.port2.bind(channel2);
    receiver.port2.bind(channel2);
  }
};

class TopTop: public sc_module {

public:

  Top1 top1;

  TopTop(sc_module_name mn):
    sc_module(mn),
    top1("top1")
  { }
};

int sc_main(int argc, char **argv)
{
  TopTop t("top");
  sc_start();

  return 0;
}
