// one_channel.cc
// 04/13/17 RD   replaced the broken channel, clean sender-receiver setup
// 04/07/17 RD   made the infinite loop terminate
// 04/07/17 ZC   initial version (based on conceptional DVD player play.cpp)

#include <stdio.h>
#include "systemc.h"

#define ITERATIONS 10
#define BUFSIZE    1ul

// queue communication

class i_long_long_sender: virtual public sc_interface
{
  public:
    virtual void send(long long d) = 0;
};

class i_long_long_receiver: virtual public sc_interface
{
  public:
    virtual long long receive(void) = 0;
};

class c_long_long_queue: public sc_channel,
        public i_long_long_sender, public i_long_long_receiver
{
  public:
    c_long_long_queue(sc_module_name name, unsigned long size):
                sc_channel(name), m_size(size)
    {
      wr = 0;
      ws = 0;
      p = 0;
      n = 0;
      buffer = new long long[size];
    }

    ~c_long_long_queue(void)
    {
      delete[] buffer;
    }

    long long receive(void)
    {
      long long d;
      while (!n)
      {
        wr ++;
        wait(r);
        wr --;
      }

      if (n <= p)
      {
        d = buffer[p - n];
      }
      else
      {
        d = buffer[p + m_size - n];
      }
      n --;

      if (ws)
      {
        s.notify(SC_ZERO_TIME);
      }

      return d;
    }

    void send(long long d)
    {
      while (n >= m_size)
      {
        ws ++;
        wait(s);
        ws --;
      }

      buffer[p] = d;
      p ++;
      if (p >= m_size)
      {
        p = 0;
      }
      n ++;

      if (wr)
      {
        r.notify(SC_ZERO_TIME);
      }
    }

  private:
    sc_event       r, s;
    unsigned long  wr;
    unsigned long  ws;
    unsigned long  p;
    unsigned long  n;
    long long      *buffer;
    unsigned long  m_size;
};

SC_MODULE(S)
{
  sc_port<i_long_long_sender> p1;
  int n;

  void main(void)
  {
    long long video = 42;

    for(int i=0; i<ITERATIONS; i++)
    {
	p1->send(video);
	n++;
    }
    p1->send(0);
    n++;
    assert(n == ITERATIONS+1);
  }

  SC_CTOR(S)
  {
    n = 0;
    SC_THREAD(main);
  }
};

SC_MODULE(R)
{
  sc_port<i_long_long_receiver> p1;
  int n;

  void main(void)
  {
    long long inFrame;

    do {
      inFrame = p1->receive();
      wait(33330, SC_US);
      putchar('V');
      fflush(stdout);
      n++;
    } while(inFrame > 0);
    printf("\n");
    assert(n == ITERATIONS+1);
  }

  SC_CTOR(R)
  {
    n = 0;
    SC_THREAD(main);
  }
};

SC_MODULE(Top)
{
  c_long_long_queue c;
  S s;
  R r;

  void before_end_of_elaboration()
  {
    s.p1.bind(c);
    r.p1.bind(c);
  }

  SC_CTOR(Top)
  : c("c", BUFSIZE)
  , s("s")
  , r("r")
  { }
};

int sc_main(int argc, char* argv[])
{
  Top top("top");
  printf("one_channel starts\n");
  assert(top.s.n == 0);
  assert(top.r.n == 0);
  sc_start();
  assert(top.s.n == ITERATIONS+1);
  assert(top.r.n == ITERATIONS+1);
  printf("one_channel done\n");
  return 0;
}

/* ex: set softtabstop=8 tabstop=8 shiftwidth=8 expandtab: */
